from numpy.testing import assert_almost_equal
import torch

from allennlp.common import Params

from allennlp_semparse.models.atis.atis_semantic_parser import AtisSemanticParser
from allennlp_semparse.parsimonious_languages.worlds import AtisWorld
from allennlp_semparse.state_machines.states import GrammarStatelet
from ... import SemparseTestCase


class TestAtisGrammarStatelet(SemparseTestCase):
    def test_atis_grammar_statelet(self):
        world = AtisWorld(
            [("give me all flights from boston to " "philadelphia next week arriving after lunch")]
        )
        action_sequence = [
            'statement -> [query, ";"]',
            'query -> ["(", "SELECT", distinct, select_results, "FROM", table_refs, '
            'where_clause, ")"]',
            'distinct -> ["DISTINCT"]',
            "select_results -> [col_refs]",
            'col_refs -> [col_ref, ",", col_refs]',
            'col_ref -> ["city", ".", "city_code"]',
            "col_refs -> [col_ref]",
            'col_ref -> ["city", ".", "city_name"]',
            "table_refs -> [table_name]",
            'table_name -> ["city"]',
            'where_clause -> ["WHERE", "(", conditions, ")"]',
            "conditions -> [condition]",
            "condition -> [biexpr]",
            'biexpr -> ["city", ".", "city_name", binaryop, city_city_name_string]',
            'binaryop -> ["="]',
            "city_city_name_string -> [\"'BOSTON'\"]",
        ]

        grammar_state = GrammarStatelet(
            ["statement"], world.valid_actions, AtisSemanticParser.is_nonterminal
        )
        for action in action_sequence:
            grammar_state = grammar_state.take_action(action)
        assert grammar_state._nonterminal_stack == []
